/*

		Comma separated value database format scanner

		This function implements a somewhat extended flavour
		of CSV.  In addition to the standard quoted fields,
		permitting embedded commas, with embedded quotes
        represented as "", backslash escaped character
		expressed as three octal digits are also permitted,
		with a double backslash representing an embedded
		backslash.	This is necessary to permit fields
		which include end-of-line delimiters which would
		otherwise truncate the record when it is read.

*/

#include "Hackdiet.h"				/* Included to obtain CSV definition.
									   You can use this file stand-alone in
									   other Pilot applications by un-commenting
									   the following two includes. */
#ifdef CSV

/*	<ctype.h> and <string.h> emulation (of the functions we need) for PalmOS.  */

#define strlen(s)	StrLen(s)

#define EOS     '\0'

static char *csptr; 				  /* CSV scan pointer */

/*	CSVSCANINIT  --  Initialise scanning of a CSV record.  */

void CSVscanInit(char *s)
{
	csptr = s;
}

/*	CSVSCANFIELD  --  Scan next field from a CSV record.
					  Returns true if field found, false at
					  end of record.  */

Boolean CSVscanField(char *f)
{
	char *ef = f;
	Boolean foundfield = false, quoted = false;

	if (*csptr != EOS) {
		foundfield = true;
		while ((*csptr != EOS) && isSpace(*csptr)) {
			csptr++;
		}
        if (*csptr == '"') {
			quoted = true;
			csptr++;
			while (*csptr != EOS) {
                if (*csptr == '"') {
                    if (csptr[1] == '"') {
                        *f++ = '"';
						csptr += 2;
					} else {
						csptr++;
						break;
					}
                } else if (*csptr == '\\') {
                    if (csptr[1] == '\\') {
                        *f++ = '\\';
						csptr += 2;
					} else if (isOctal(csptr[1]) && isOctal(csptr[2]) && isOctal(csptr[3])) {
						unsigned int v;

//                      sscanf(csptr + 1, "%3o", &v);
						v = ((csptr[1] - '0') << 6) | ((csptr[2] - '0') << 3) | (csptr[3] - '0');
						*f++ = (char) v;
						csptr += 4;
					} else {
						*f++ = csptr[1];
						csptr += 2;
					}
				} else {
					*f++ = *csptr++;
				}
			}
		}
        while (*csptr != ',' && *csptr != EOS) {
			*f++ = *csptr++;
		}
        if (*csptr == ',') {
			csptr++;
		}
	}
	*f++ = EOS;
	if (foundfield && !quoted) {
		while (strlen(ef) > 0 && isSpace(ef[strlen(ef) - 1])) {
			ef[strlen(ef) - 1] = EOS;
		}
	}
	return foundfield;
}

//	CSVRECORDINIT  --  Initialise extraction of records from in-memory CSV file

static char *recptr;
static int reclen, ri;

void csvRecordInit(char *r, int l)
{
	recptr = r;
	reclen = l;
	ri = 0;
}

/*	CSVRECORDNEXT  --  Return next record from in-memory text file.  Returns
					   false when end of file reached.  Truncates records as
					   required.  Understands PC, Mac, Unix, and VAX end of
					   line conventions.  */
					   
Boolean csvRecordNext(char *dest, int destl)
{
#define	isEOL(c)	(((c) == '\r') || ((c) == '\n'))
	int rend, rl;
	
	assert(destl > 0);
	if (ri >= reclen) {
		return false;
	}
	
	for (rend = ri; (rend < reclen) && (!isEOL(recptr[rend])); rend++)
		;
		
	rl = rend - ri;
	if (rl == 0) {
		*dest = 0;
	} else {
		if (rl < (destl - 1)) {
			MemMove(dest, recptr + ri, rl);
			dest[rl] = 0;
		} else {
			MemMove(dest, recptr + ri, destl - 1);
			dest[destl - 1] = 0;
		}
	}
	ri = rend + 1;
	if ((ri < reclen) && (isEOL(recptr[ri]) && (recptr[ri - 1] != recptr[ri]))) {
		ri++;
	}
	return true;
}

//	CSVRECORDCOUNT  --  Return number of records remaining in in-memory monolithic file

UInt16 csvRecordCount(void)
{
	int rp, rend;
	UInt16 rcount = 0;
	
	rp = ri;
	while (rp < reclen) {
		for (rend = rp; (rend < reclen) && (!isEOL(recptr[rend])); rend++)
			;
			
		rp = rend + 1;
		if ((rp < reclen) && (isEOL(recptr[rp]) && (recptr[rp - 1] != recptr[rp]))) {
			rp++;
		}
		rcount++;
	}
	
	return rcount;
}
					  
#endif